/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "ModulesPicker.h"

#include <Core/ModulesManager/ModulesManager.h>
#include <Modules/Splitter/Splitter.h>

#include <QDebug>
#include <QPushButton>

#include "ui_ModulesPicker.h"

ModulesPicker::ModulesPicker() : QDialog(), ui(new Ui::ModulesPicker)
{
    ui->setupUi(this);
    createAndConnectUI();

    setAttribute(Qt::WA_DeleteOnClose,
                 true);  // WA_DeleteOnClose is set to true, so this
                         // widget will be deleted on close
}

ModulesPicker::~ModulesPicker() { delete ui; }

void ModulesPicker::createAndConnectUI()
{
    connect(ui->button_createModule, &QPushButton::clicked, this,
            &ModulesPicker::onCreateModuleButtonClick);

    connect(ui->pushButton_horizontal, &QPushButton::clicked, this,
            [=]() { createSplitter(Qt::Horizontal); });
    connect(ui->pushButton_vertical, &QPushButton::clicked, this,
            [=]() { createSplitter(Qt::Vertical); });

    /*  Important:
     *  The text of this buttons is set to be the name of the module to load
     * onclick
     */
    for (ModuleInfo moduleInfo : ModulesManager::getModulesInfo())
    {
        switch (moduleInfo.getModuleCategory())
        {
            case ModuleCategory::DEFAULT:
                ui->comboBox_modulesList->addItem(moduleInfo.getModuleName());
                break;
            case ModuleCategory::DATASOURCE:
                ui->data_source_layout->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::DATAVISUAL:
                ui->data_visual_layout->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::UTILITY:
                ui->utility_layout->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::HOME:
                ui->home_layout->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            default:
                break;
        }
    }
}

void ModulesPicker::connectSlot(QPushButton* button)
{
    connect(button, &QPushButton::clicked, this,
            [this, button]()
            {
                Module* module =
                    ModulesManager::instantiateModuleByName(button->text());
                if (module != nullptr)
                {
                    onModuleSelected(module);
                }
            });
}

void ModulesPicker::onCreateModuleButtonClick()
{
    QString moduleName = ui->comboBox_modulesList->currentText();
    onModuleSelected(moduleName);
}

void ModulesPicker::createSplitter(Qt::Orientation orientation)
{
    Splitter* module = qobject_cast<Splitter*>(
        ModulesManager::instantiateModuleById(ModuleId::SPLITTER));
    if (module)
    {
        module->setOrientation(orientation);
        onModuleSelected(module);
    }
}

QPushButton* ModulesPicker::createButton(const QString& name)
{
    QPushButton* button = new QPushButton(name);
    connect(button, &QPushButton::clicked, this,
            [this, button]()
            {
                if (button != nullptr)
                {
                    onModuleSelected(button->text());
                }
            });

    return button;
}

void ModulesPicker::onModuleSelected(const QString& mName)
{
    Module* module = ModulesManager::instantiateModuleByName(mName);
    if (module != nullptr)
        onModuleSelected(module);
}

void ModulesPicker::onModuleSelected(Module* module)
{
    *selectedModule = module;
    emit moduleSelected(module);
    this->close();
}

Module* ModulesPicker::start()
{
    Module* mSelected = nullptr;
    // cppcheck-suppress danglingLifetime
    selectedModule = &mSelected;
    this->exec();
    return mSelected;
}
