/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "ModulesManager.h"

#include <Components/ModulesPicker/ModulesPicker.h>
#include <Components/SaveConfigurationDialog/SaveConfigurationDialog.h>
#include <Core/Module/Module.h>
#include <Core/XmlObject.h>
#include <Modules/SkywardHubStrings.h>

#include <QDebug>
#include <QDir>
#include <QInputDialog>
#include <QMessageBox>
#include <QStandardPaths>

ModulesList ModulesManager::modulesListHandler;

ModulesManager::ModulesManager() {}

ModulesManager::~ModulesManager()
{
    clearPages();
    for (int i = 0; i < menuActions.count(); i++)
    {
        delete menuActions[i];
    }
    menuActions.clear();
}

QString ModulesManager::getModuleName(ModuleId id)
{
    if (ModulesManager::modulesListHandler.containsId(id))
        return ModulesManager::modulesListHandler.getModuleName(id);

    return "";
}

Module* ModulesManager::instantiateModuleById(ModuleId id)
{
    if (ModulesManager::modulesListHandler.containsId(id))
        return ModulesManager::modulesListHandler.getFactory(id)();

    return nullptr;
}

Module* ModulesManager::instantiateModuleByName(const QString& moduleName)
{
    return ModulesManager::modulesListHandler.findFactoryByModuleName(
        moduleName)();
}

QList<Module*> ModulesManager::loadModuleFromXml(XmlObject& xml)
{
    QList<Module*> windows;

    for (int i = 0; i < xml.childCount(); i++)
    {
        XmlObject* child   = xml.getChild(i);
        QString moduleName = child->getObjectName();

        Module* module = ModulesManager::instantiateModuleByName(moduleName);

        if (module != nullptr)
        {
            module->fromXmlObject(*child);
            windows.append(module);
        }
    }

    return windows;
}

void ModulesManager::setModules(QList<Module*> modules)
{
    int i;
    for (i = 0; i < pages.count() && i < modules.count(); i++)
    {
        Module* old = pages[i];
        emit pages[i]->getEventHandler()->replaceMeWith(old, modules[i]);
    }
    if (i < pages.count())
    {
        for (int c = pages.count() - 1; c > i; c--)
        {
            delete pages[c];
        }
    }
    for (; i < modules.count(); i++)
    {
        addPage(modules[i]);
    }
}

void ModulesManager::addModules(QList<Module*> modules)
{
    for (int i = 0; i < modules.count(); i++)
    {
        addPage(modules[i]);
    }
}

void ModulesManager::addPage(Module* module)
{
    pages.append(module);
    connectModule(module);
    emit pageAdded(pages.indexOf(module));
}

void ModulesManager::removePage(Module* module)
{
    int index = pages.indexOf(module);
    if (index >= 0)
    {
        pages.removeAt(index);
        disconnectModule(module);
    }
}

Module* ModulesManager::instantiateDefaultModule()
{
    return ModulesManager::instantiateModuleById(ModuleId::EMPTY);
}

Module* ModulesManager::invokeModulesPickerService()
{
    ModulesPicker* p = new ModulesPicker();
    return p->start();
}

QList<QString> ModulesManager::getModulesNamesList()
{
    return ModulesManager::modulesListHandler.getModulesNamesList();
}

void ModulesManager::connectModule(Module* module)
{
    if (module)
    {
        connect(module->getEventHandler(), &EventHandler::contextMenuRequest,
                this, &ModulesManager::onContextMenuRequest);
        connect(module->getEventHandler(), &EventHandler::beforeDelete, this,
                &ModulesManager::onModuleDeleted);
        connect(module->getEventHandler(), &EventHandler::replaceMeWith, this,
                &ModulesManager::onReplaceMeWith);
    }

    // connect(module->getEventHandler(),&EventHandler::replaceMeWithModules,this,&ModulesManager::openWindows);
}

void ModulesManager::disconnectModule(Module* module)
{
    if (module)
    {
        disconnect(module->getEventHandler(), &EventHandler::contextMenuRequest,
                   this, &ModulesManager::onContextMenuRequest);
        disconnect(module->getEventHandler(), &EventHandler::beforeDelete, this,
                   &ModulesManager::onModuleDeleted);
        disconnect(module->getEventHandler(), &EventHandler::replaceMeWith,
                   this, &ModulesManager::onReplaceMeWith);
    }
}

void ModulesManager::onModuleDeleted(Module* deletedModule)
{
    emit deletedModule->getEventHandler()->replaceMeWith(
        deletedModule, instantiateDefaultModule());
}

void ModulesManager::openNewEmptyWindow()
{
    addPage(ModulesManager::instantiateDefaultModule());
}

Module* ModulesManager::getModuleAt(int index)
{
    if (index >= 0 && index < pages.count())
    {
        return pages.at(index);
    }
    return nullptr;
}

int ModulesManager::getModuleCount() { return pages.count(); }

void ModulesManager::clearPages()
{
    for (int i = pages.count() - 1; i >= 0; i--)
    {
        delete pages[i];
    }
    pages.clear();
}

void ModulesManager::onReplaceMeWith(Module* sender, Module* newModule)
{
    if (sender != nullptr && newModule != nullptr)
    {
        int index = pages.indexOf(sender);
        if (index >= 0)
        {
            pages[index] = newModule;
            connectModule(newModule);
            disconnectModule(sender);
        }
    }
}

void ModulesManager::onContextMenuRequest(QMenu& menu, const QPoint& p)
{
    QMenu hubMenu("Global menu");
    hubMenu.addActions(getHubMenuActions());
    menu.addMenu(&hubMenu);
    emit contextMenuRequest(menu, p);
}

QList<QAction*> ModulesManager::getHubMenuActions()
{
    if (menuActions.count() == 0)
    {
        QAction* save = new QAction("Save");
        menuActions.append(save);
        connect(save, &QAction::triggered, this,
                &ModulesManager::openConfigurationWindow);

        QAction* openWindows = new QAction("New Window");
        menuActions.append(openWindows);
        connect(openWindows, &QAction::triggered, this,
                &ModulesManager::openNewEmptyWindow);
    }

    return menuActions;
}

QList<ModuleInfo> ModulesManager::getModulesInfo()
{
    return modulesListHandler.getModulesInfo();
}

void ModulesManager::openConfigurationWindow()
{
    if (pages.isEmpty())
    {
        QMessageBox errorMsg;
        errorMsg.setText("No module to be saved found");
        errorMsg.exec();
        return;
    }

    // Ask configuration data to the user
    SaveConfigurationDialog* saveDialog = new SaveConfigurationDialog();
    saveDialog->setConfigName(SkywardHubStrings::defaultConfigurationFileName);
    saveDialog->setConfigIconPath(
        SkywardHubStrings::defaultConfigurationIconPath);
    QString configName;
    QString configIconPath;
    QString description;
    saveDialog->pickConfigData(&configName, &configIconPath, &description);

    XmlObject xml(SkywardHubStrings::configurationNameValue);
    xml.addAttribute(SkywardHubStrings::configurationIconPathAttribute,
                     configIconPath);
    xml.addAttribute(SkywardHubStrings::configurationDescriptionAttribute,
                     description);

    if (!configName.isEmpty())
    {
        configName = configName.replace(".xml", "");
        xml.addAttribute(SkywardHubStrings::configurationNameAttribute,
                         configName);
        if (configName.indexOf(".xml") < 0)
        {
            configName.append(".xml");
        }

        if (!QDir(SkywardHubStrings::defaultPrefabsFolder).exists())
        {
            QDir().mkpath(SkywardHubStrings::defaultPrefabsFolder);
        }

        for (Module* module : pages)
        {
            xml.addChild(module->toXmlObject());
        }

        QString resultTxt = "Impossible to save";
        if (xml.writeToFile(SkywardHubStrings::defaultPrefabsFolder +
                            configName))
        {
            resultTxt = "The configuration has been saved in " + configName;
        }
        QMessageBox resultMsg;
        resultMsg.setText(resultTxt);
        resultMsg.exec();
    }
}
